@@ -16,11 +16,20 @@
 """
 
 import re
+import sys
 import types
 
-core_types = [ ]
+core_types = []
 
-class Error(Exception):
+if sys.version_info[0] == 2:
+    string_types = (str, unicode)
+    num_types = (float, long, int)
+else:
+    string_types = (str,)
+    num_types = (float, int)
+
+
+class RxError(Exception):
     pass
 
 class Util(object):
@@ -34,10 +43,14 @@ class Util(object):
             range[entry] = opt[entry]
 
         def check_range(value):
-            if range.get('min'   ) != None and value <  range['min'   ]: return False
-            if range.get('min-ex') != None and value <= range['min-ex']: return False
-            if range.get('max-ex') != None and value >= range['max-ex']: return False
-            if range.get('max'   ) != None and value >  range['max'   ]: return False
+            if range.get('min'   ) != None and value <  range['min'   ]:
+                return False
+            if range.get('min-ex') != None and value <= range['min-ex']:
+                return False
+            if range.get('max-ex') != None and value >= range['max-ex']:
+                return False
+            if range.get('max'   ) != None and value >  range['max'   ]:
+                return False
             return True
 
         return check_range
@@ -54,18 +67,20 @@ class Factory(object):
             for t in core_types: self.register_type(t)
 
     @staticmethod
-    def _default_prefixes(): pass
+    def _default_prefixes():
+        pass
 
     def expand_uri(self, type_name):
-        if re.match('^\w+:', type_name): return type_name
+        if re.match('^\w+:', type_name):
+            return type_name
 
         m = re.match('^/([-._a-z0-9]*)/([-._a-z0-9]+)$', type_name)
 
         if not m:
-            raise ValueError("couldn't understand type name '%s'" % type_name)
+            raise RxError("couldn't understand type name '%s'" % type_name)
 
         if not self.prefix_registry.get(m.group(1)):
-            raise ValueError(
+            raise RxError(
                 "unknown prefix '%s' in type name '%s'" % (m.group(1), type_name)
             )
 
@@ -73,7 +88,7 @@ class Factory(object):
 
     def add_prefix(self, name, base):
         if self.prefix_registry.get(name, None):
-            raise Error("the prefix '%s' is already registered" % name)
+            raise RxError("the prefix '%s' is already registered" % name)
 
         self.prefix_registry[name] = base;
 
@@ -87,7 +102,7 @@ class Factory(object):
 
     def learn_type(self, uri, schema):
         if self.type_registry.get(uri, None):
-            raise Error("tried to learn type for already-registered uri %s" % uri)
+            raise RxError("tried to learn type for already-registered uri %s" % uri)
 
         # make sure schema is valid
         # should this be in a try/except?
@@ -96,21 +111,22 @@ class Factory(object):
         self.type_registry[uri] = { "schema": schema }
 
     def make_schema(self, schema):
-        if type(schema) in (str, unicode):
+        if isinstance(schema, string_types):
             schema = { "type": schema }
 
         if not type(schema) is dict:
-            raise Error('invalid schema argument to make_schema')
+            raise RxError('invalid schema argument to make_schema')
 
         uri = self.expand_uri(schema["type"])
 
-        if not self.type_registry.get(uri): raise Error("unknown type %s" % uri)
+        if not self.type_registry.get(uri):
+            raise RxError("unknown type %s" % uri)
 
         type_class = self.type_registry[ uri ]
 
         if type(type_class) is dict:
             if not set(schema.keys()).issubset(set(['type'])):
-                raise Error('composed type does not take check arguments');
+                raise RxError('composed type does not take check arguments');
             return self.make_schema(type_class["schema"])
         else:
             return type_class(schema, self)
@@ -122,62 +138,70 @@ class _CoreType(object):
 
     def __init__(self, schema, rx):
         if not set(schema.keys()).issubset(set(['type'])):
-            raise Error('unknown parameter for //%s' % self.subname())
+            raise RxError('unknown parameter for //%s' % self.subname())
 
-    def check(self, value): return False
+    def check(self, value):
+        return False
 
 class AllType(_CoreType):
     @staticmethod
-    def subname(): return 'all'
+    def subname():
+        return 'all'
 
     def __init__(self, schema, rx):
         if not set(schema.keys()).issubset(set(('type', 'of'))):
-            raise Error('unknown parameter for //all')
+            raise RxError('unknown parameter for //all')
         
         if not(schema.get('of') and len(schema.get('of'))):
-            raise Error('no alternatives given in //all of')
+            raise RxError('no alternatives given in //all of')
 
         self.alts = [ rx.make_schema(s) for s in schema['of'] ]
 
     def check(self, value):
         for schema in self.alts:
-            if (not schema.check(value)): return False
+            if (not schema.check(value)):
+                return False
         return True
 
 class AnyType(_CoreType):
     @staticmethod
-    def subname(): return 'any'
+    def subname():
+        return 'any'
 
     def __init__(self, schema, rx):
         self.alts = None
 
         if not set(schema.keys()).issubset(set(('type', 'of'))):
-            raise Error('unknown parameter for //any')
+            raise RxError('unknown parameter for //any')
         
         if schema.get('of') != None:
-            if not schema['of']: raise Error('no alternatives given in //any of')
+            if not schema['of']:
+                raise RxError('no alternatives given in //any of')
             self.alts = [ rx.make_schema(alt) for alt in schema['of'] ]
 
     def check(self, value):
-        if self.alts is None: return True
+        if self.alts is None:
+            return True
 
         for alt in self.alts:
-            if alt.check(value): return True
+            if alt.check(value):
+                return True
 
         return False
 
 class ArrType(_CoreType):
     @staticmethod
-    def subname(): return 'arr'
+    def subname():
+        return 'arr'
 
     def __init__(self, schema, rx):
         self.length = None
 
         if not set(schema.keys()).issubset(set(('type', 'contents', 'length'))):
-            raise Error('unknown parameter for //arr')
+            raise RxError('unknown parameter for //arr')
 
         if not schema.get('contents'):
-            raise Error('no contents provided for //arr')
+            raise RxError('no contents provided for //arr')
 
         self.content_schema = rx.make_schema(schema['contents'])
 
@@ -185,102 +209,124 @@ class ArrType(_CoreType):
             self.length = Util.make_range_check( schema["length"] )
 
     def check(self, value):
-        if not(type(value) in [ type([]), type(()) ]): return False
-        if self.length and not self.length(len(value)): return False
+        if not(type(value) in [ type([]), type(()) ]):
+            return False
+        if self.length and not self.length(len(value)):
+            return False
 
         for item in value:
-            if not self.content_schema.check(item): return False
+            if not self.content_schema.check(item):
+                return False
 
         return True;
 
 class BoolType(_CoreType):
     @staticmethod
-    def subname(): return 'bool'
+    def subname():
+        return 'bool'
 
     def check(self, value):
-        if value is True or value is False: return True
+        if value is True or value is False:
+            return True
         return False
 
 class DefType(_CoreType):
     @staticmethod
-    def subname(): return 'def'
+    def subname():
+        return 'def'
 
-    def check(self, value): return not(value is None)
+    def check(self, value):
+        return not(value is None)
 
 class FailType(_CoreType):
     @staticmethod
-    def subname(): return 'fail'
+    def subname():
+        return 'fail'
 
-    def check(self, value): return False
+    def check(self, value):
+        return False
 
 class IntType(_CoreType):
     @staticmethod
-    def subname(): return 'int'
+    def subname():
+        return 'int'
 
     def __init__(self, schema, rx):
         if not set(schema.keys()).issubset(set(('type', 'range', 'value'))):
-            raise Error('unknown parameter for //int')
+            raise RxError('unknown parameter for //int')
 
         self.value = None
-        if schema.has_key('value'):
-            if not type(schema['value']) in (float, int, long):
-                raise Error('invalid value parameter for //int')
+        if 'value' in schema:
+            if not isinstance(schema['value'], num_types) or \
+               isinstance(schema['value'], bool):
+                raise RxError('invalid value parameter for //int')
             if schema['value'] % 1 != 0:
-                raise Error('invalid value parameter for //int')
+                raise RxError('invalid value parameter for //int')
             self.value = schema['value']
 
         self.range = None
-        if schema.has_key('range'):
+        if 'range' in schema:
             self.range = Util.make_range_check( schema["range"] )
 
     def check(self, value):
-        if not(type(value) in (float, int, long)): return False
-        if value % 1 != 0: return False
-        if self.range and not self.range(value): return False
-        if (not self.value is None) and value != self.value: return False
+        if not isinstance(value, num_types) or isinstance(value, bool):
+            return False
+        if value % 1 != 0:
+            return False
+        if self.range and not self.range(value):
+            return False
+        if (not self.value is None) and value != self.value:
+            return False
         return True
 
 class MapType(_CoreType):
     @staticmethod
-    def subname(): return 'map'
+    def subname():
+        return 'map'
 
     def __init__(self, schema, rx):
         self.allowed = set()
 
         if not set(schema.keys()).issubset(set(('type', 'values'))):
-            raise Error('unknown parameter for //map')
+            raise RxError('unknown parameter for //map')
 
         if not schema.get('values'):
-            raise Error('no values given for //map')
+            raise RxError('no values given for //map')
 
         self.value_schema = rx.make_schema(schema['values'])
 
     def check(self, value):
-        if not(type(value) is type({})): return False
+        if not(type(value) is type({})):
+            return False
 
         for v in value.values():
-            if not self.value_schema.check(v): return False
+            if not self.value_schema.check(v):
+                return False
 
         return True;
 
 class NilType(_CoreType):
     @staticmethod
-    def subname(): return 'nil'
+    def subname():
+        return 'nil'
 
-    def check(self, value): return value is None
+    def check(self, value):
+        return value is None
 
 class NumType(_CoreType):
     @staticmethod
-    def subname(): return 'num'
+    def subname():
+        return 'num'
 
     def __init__(self, schema, rx):
         if not set(schema.keys()).issubset(set(('type', 'range', 'value'))):
-            raise Error('unknown parameter for //num')
+            raise RxError('unknown parameter for //num')
 
         self.value = None
-        if schema.has_key('value'):
-            if not type(schema['value']) in (float, int, long):
-                raise Error('invalid value parameter for //num')
+        if 'value' in schema:
+            if not isinstance(schema['value'], num_types) or \
+               isinstance(schema['value'], bool):
+                raise RxError('invalid value parameter for //num')
             self.value = schema['value']
 
         self.range = None
@@ -289,37 +335,44 @@ class NumType(_CoreType):
             self.range = Util.make_range_check( schema["range"] )
 
     def check(self, value):
-        if not(type(value) in (float, int, long)): return False
-        if self.range and not self.range(value): return False
-        if (not self.value is None) and value != self.value: return False
+        if not isinstance(value, num_types) or isinstance(value, bool):
+            return False
+        if self.range and not self.range(value):
+            return False
+        if (not self.value is None) and value != self.value:
+            return False
         return True
 
 class OneType(_CoreType):
     @staticmethod
-    def subname(): return 'one'
+    def subname():
+        return 'one'
 
     def check(self, value):
-        if type(value) in (int, float, long, bool, str, unicode): return True
+        if isinstance(value, num_types + string_types + (bool,)):
+            return True
 
         return False
 
 class RecType(_CoreType):
     @staticmethod
-    def subname(): return 'rec'
+    def subname():
+        return 'rec'
 
     def __init__(self, schema, rx):
         if not set(schema.keys()).issubset(set(('type', 'rest', 'required', 'optional'))):
-            raise Error('unknown parameter for //rec')
+            raise RxError('unknown parameter for //rec')
 
         self.known = set()
         self.rest_schema = None
-        if schema.get('rest'): self.rest_schema = rx.make_schema(schema['rest'])
+        if schema.get('rest'):
+            self.rest_schema = rx.make_schema(schema['rest'])
 
         for which in ('required', 'optional'):
             self.__setattr__(which, { })
             for field in schema.get(which, {}).keys():
                 if field in self.known:
-                    raise Error('%s appears in both required and optional' % field)
+                    raise RxError('%s appears in both required and optional' % field)
 
                 self.known.add(field)
 
@@ -328,39 +381,49 @@ class RecType(_CoreType):
                 )
 
     def check(self, value):
-        if not(type(value) is type({})): return False
+        if not(type(value) is type({})):
+            return False
 
         unknown = [ ]
         for field in value.keys():
-            if not field in self.known: unknown.append(field)
+            if not field in self.known:
+                unknown.append(field)
 
-        if len(unknown) and not self.rest_schema: return False
+        if len(unknown) and not self.rest_schema:
+            return False
 
         for field in self.required.keys():
-            if not value.has_key(field): return False
-            if not self.required[field].check( value[field] ): return False
+            if field not in value:
+                return False
+            if not self.required[field].check( value[field] ):
+                return False
 
         for field in self.optional.keys():
-            if not value.has_key(field): continue
-            if not self.optional[field].check( value[field] ): return False
+            if field not in value:
+                continue
+            if not self.optional[field].check( value[field] ):
+                return False
 
         if len(unknown):
             rest = { }
-            for field in unknown: rest[field] = value[field]
-            if not self.rest_schema.check(rest): return False
+            for field in unknown:
+                rest[field] = value[field]
+            if not self.rest_schema.check(rest):
+                return False
 
         return True;
 
 class SeqType(_CoreType):
     @staticmethod
-    def subname(): return 'seq'
+    def subname():
+        return 'seq'
 
     def __init__(self, schema, rx):
         if not set(schema.keys()).issubset(set(('type', 'contents', 'tail'))):
-            raise Error('unknown parameter for //seq')
+            raise RxError('unknown parameter for //seq')
 
         if not schema.get('contents'):
-            raise Error('no contents provided for //seq')
+            raise RxError('no contents provided for //seq')
 
         self.content_schema = [ rx.make_schema(s) for s in schema["contents"] ]
 
@@ -369,7 +432,8 @@ class SeqType(_CoreType):
             self.tail_schema = rx.make_schema(schema['tail'])
 
     def check(self, value):
-        if not(type(value) in [ type([]), type(()) ]): return False
+        if not(type(value) in [ type([]), type(()) ]):
+            return False
 
         if len(value) < len(self.content_schema):
             return False
@@ -379,35 +443,40 @@ class SeqType(_CoreType):
                 return False
 
         if len(value) > len(self.content_schema):
-            if not self.tail_schema: return False
+            if not self.tail_schema:
+                return False
 
-            if not self.tail_schema.check(value[ len(self.content_schema) :  ]):
+            if not self.tail_schema.check(value[len(self.content_schema):]):
                 return False
 
         return True;
 
 class StrType(_CoreType):
     @staticmethod
-    def subname(): return 'str'
+    def subname():
+        return 'str'
 
     def __init__(self, schema, rx):
         if not set(schema.keys()).issubset(set(('type', 'value', 'length'))):
-            raise Error('unknown parameter for //str')
+            raise RxError('unknown parameter for //str')
 
         self.value = None
-        if schema.has_key('value'):
-            if not type(schema['value']) in (str, unicode):
-                raise Error('invalid value parameter for //str')
+        if 'value' in schema:
+            if not isinstance(schema['value'], string_types):
+                raise RxError('invalid value parameter for //str')
             self.value = schema['value']
 
         self.length = None
-        if schema.has_key('length'):
+        if 'length' in schema:
             self.length = Util.make_range_check( schema["length"] )
 
     def check(self, value):
-        if not type(value) in (str, unicode): return False
-        if (not self.value is None) and value != self.value: return False
-        if self.length and not self.length(len(value)): return False
+        if not isinstance(value, string_types):
+            return False
+        if (not self.value is None) and value != self.value:
+            return False
+        if self.length and not self.length(len(value)):
+            return False
         return True
 
 core_types = [
